/* -*-C++-*- */
/*
 * Phase-type fitting.
 *
 * Copyright (C) 2004, 2005 Carnegie Mellon University
 *
 * This file is part of Tempastic-DTP.
 *
 * Tempastic-DTP is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Tempastic-DTP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Tempastic-DTP; if not, write to the Free Software
 * Foundation, Inc., #59 Temple Place, Suite 330, Boston, MA
 * 02111-1307 USA
 *
 * $Id: phfit.h,v 2.1 2005/02/08 11:39:13 lorens Exp $
 */
#ifndef PHFIT_H
#define PHFIT_H

#include "distributions.h"


/* ====================================================================== */
/* TransitionRateMap */

/*
 * A phase transition rate map.
 */
struct TransitionRateMap : public std::map<std::pair<size_t, size_t>, double> {
};


/* ====================================================================== */
/* PHParameters */

/*
 * Parameters defining a continuous phase-type distribution.
 */
struct PHParameters {
  /* Number of phases. */
  size_t n;
  /* Phase transition rates. */
  TransitionRateMap rates;

  /* Adds a transition rate. */
  void add_rate(size_t i, size_t j, double rate) {
    rates.insert(std::make_pair(std::make_pair(i, j), rate));
  }
};


/* ====================================================================== */
/* Phase-type fitting methods. */

/* Provides the parameters for a continuous phase-type distribution
   matching the first n moments of this distribution. */
void ph_matching_moments(PHParameters& params,
			 const Distribution& dist, size_t n);

/* Provides the parameters for a continuous phase-type distribution
   using the EM algorithm with n phases. */
void ph_em_fit(PHParameters& params, const Distribution& dist, size_t n,
	       double discr, double epsilon, size_t iters);


#endif /* PHFIT_H */
